import os
import torch
import torch.nn as nn
import warnings
from aiia.model.Model import AIIAConfig, AIIABase
from transformers import PreTrainedModel
from .config import aiuNNConfig
import warnings

class aiuNN(PreTrainedModel):
    config_class = aiuNNConfig
    def __init__(self, config: aiuNNConfig, base_model: PreTrainedModel = None):
        super().__init__(config)
        self.config = config
        if base_model is not None:
            # Only copy submodules if base_model is provided
            self.base_layers = nn.Sequential(*[layer for layer in base_model.cnn])
        else:
            # At inference, modules will be loaded from state_dict
            self.base_layers = self._build_base_layers_from_config(config)

        scale_factor = self.config.upsample_scale
        out_channels = self.config.num_channels * (scale_factor ** 2)
        self.pixel_shuffle_conv = nn.Conv2d(
            in_channels=self.config.hidden_size,
            out_channels=out_channels,
            kernel_size=self.config.kernel_size,
            padding=1
        )
        self.pixel_shuffle = nn.PixelShuffle(scale_factor)

    def _build_base_layers_from_config(self, config):
        """
        Reconstruct the base layers (e.g., CNN) using only the config.
        This must match exactly how your base model builds its layers!
        """
        layers = []
        in_channels = config.num_channels
        for _ in range(config.num_hidden_layers):
            layers.extend([
                nn.Conv2d(in_channels, config.hidden_size,
                          kernel_size=config.kernel_size, padding=1),
                getattr(nn, config.activation_function)(),
                nn.MaxPool2d(kernel_size=1, stride=1)
            ])
            in_channels = config.hidden_size
        return nn.Sequential(*layers)

    def forward(self, x):
        if self.base_layers is not None:
            x = self.base_layers(x)
        x = self.pixel_shuffle_conv(x)
        x = self.pixel_shuffle(x)
        return x



if __name__ == "__main__":
    from aiia import AIIABase, AIIAConfig
    # Create a configuration and build a base model.
    config = AIIAConfig()
    ai_config = aiuNNConfig()
    base_model = AIIABase(config)
    # Instantiate Upsampler from the base model (works correctly).
    upsampler = aiuNN(config=ai_config, base_model=base_model)
    # Save the model (both configuration and weights).
    upsampler.save_pretrained("aiunn")

    # Now load using the overridden load method; this will load the complete model.
    upsampler_loaded = aiuNN.from_pretrained("aiunn")
    print("Updated configuration:", upsampler_loaded.config.__dict__)
